/*
 * Dziaajcy printf dla SDLa
 * (c) 2002 Marek "March" Szyprowski <marek@amiga.pl>
 * Wersja 1.3 (29.10.2002)
 *
 * Historia:
 * 29.10.2002 - v1.3 - dodanie defaultowego fonta, wkompilowywanego w program
 * 04.05.2002 - v1.2 - dodanie %e w moim vsnprintf
 * 03.05.2002 - v1.1 - dodanie protezy dla MinGW32 (brak vsnprintf w bibliotece)
 * 01.05.2002 - v1.0 - pierwsza dzialajaca wersja
 *
 *
 * Niniejszy fragment kodu jest wolnym oprogramowaniem; moesz go
 * rozprowadza dalej i/lub modyfikowa na warunkach Powszechnej
 * Licencji Publicznej GNU, wydanej przez Fundacj Wolnego
 * Oprogramowania - wedug wersji 2 tej Licencji lub (wedug twojego
 * wyboru) ktrej z pniejszych wersji.
 *
 * Niniejszy program rozpowszechniany jest z nadziej, i bdzie on
 * uyteczny - jednak BEZ JAKIEJKOLWIEK GWARANCJI, nawet domylnej
 * gwarancji PRZYDATNOCI HANDLOWEJ albo PRZYDATNOCI DO OKRELONYCH
 * ZASTOSOWA. W celu uzyskania bliszych informacji signij do
 * Powszechnej Licencji Publicznej GNU.
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <SDL/SDL.h>
#include "SDL_font.h"

#ifndef SDL_DONT_USEDEFAULTFONT
#include "SDL_deffont.h"
#endif

#ifdef __MINGW32__
		 
void vsnprintf(char *buf, int len, const char *fmt, va_list ap)
{
	char *p;
	
	char *sval;
	int ival;
	double fval;

	char tmpbuf[10], *tmp=tmpbuf;
 
	for (p=fmt; *p; p++)
	{
		tmp = tmpbuf;
		if (*p != '%')
		{
			*buf++ = *p;
			continue;
		}
		while ((*tmp++ = *p++))
			if (*(tmp-1) == 'd' || *(tmp-1) == 's' || *(tmp-1) == 'f' || *(tmp-1) == 'e')
				break;
		*tmp = '\0';

		switch (*--p)
		{
			case 'd':
				ival = va_arg(ap, int);
				buf += sprintf(buf, tmpbuf, ival);
				break;
			case 's':
				sval = va_arg(ap, char *);
				buf += sprintf(buf, tmpbuf, sval);
				break;
			case 'f':
				fval = va_arg(ap, double);
				buf += sprintf(buf, tmpbuf, fval);
				break;
			case 'e':
				fval = va_arg(ap, double);
				buf += sprintf(buf, tmpbuf, fval);
				break;
			default:
				*buf++ = *p;
				break;
		}
	}
	*buf = '\0';
}

#endif

#ifndef SDL_DONT_USEDEFAULTFONT

static void unpack (unsigned char *dst, unsigned char *src, int len) 
{
	int i=0;
	unsigned char *p=src;
	unsigned char *q=dst;

	while (i < len)
	{
		if (*p >= KOD)
		{
			int c = *(p+1);
			while (c-- > 0)
				*q++ = *p - KOD;
			i += *(p+1);
			p+=2;
		}
		else
		{
			*q++ = *p;
			i++; p++;
		}
	}
}

static SDL_Surface *SDL_SetupDefault (void)
{
	SDL_Surface *s;

	s = SDL_CreateRGBSurface(SDL_SWSURFACE, DF_WIDTH, DF_HEIGHT, 8, 0, 0, 0, 0);
	SDL_SetColors(s, font_gfx_cmap, 0, DF_COLORS);
	unpack(s->pixels, font_gfx_packed_data, DF_WIDTH * DF_HEIGHT);
	return s;
}
#else
static SDL_Surface *SDL_SetupDefault()
{
	return NULL;
}
#endif

SDL_Font *SDL_LoadFont(SDL_Surface *dstdisplay, char *fontfile)
{
	SDL_Surface *tmp_srf;
	SDL_Font *fontp;
	int width, height;
	
	if (fontfile == NULL || (tmp_srf = SDL_LoadBMP(fontfile)) == NULL)
		tmp_srf = SDL_SetupDefault();

	if (tmp_srf == NULL)
		return NULL;
	
	height = tmp_srf->h;
	width = tmp_srf->w/224;
	
	fontp = malloc(sizeof(SDL_Font));
	fontp->width = width;	fontp->height = height;
	fontp->chars = SDL_ConvertSurface(tmp_srf, dstdisplay->format, SDL_SWSURFACE);
	SDL_FreeSurface(tmp_srf);
	return fontp;
}

void SDL_FreeFont(SDL_Font *font)
{
	SDL_FreeSurface(font->chars);
	free(font);
}

int SDL_DrawChar(SDL_Surface *dest, int x, int y, SDL_Font *font, unsigned char c)
{
	SDL_Rect srcrect, dstrect;

	if (c > 256 || c < 32)
		return 0;

	srcrect.x = (c - 32) * font->width; srcrect.y = 0;
	srcrect.w = font->width; srcrect.h = font->height;
	dstrect.x = x;	dstrect.y = y;

	SDL_BlitSurface(font->chars, &srcrect, dest, &dstrect);
	return 1;
}

void SDL_vsPrintF(SDL_Surface *dest, int x, int y, SDL_Font *font, const char *fmt, va_list ap)
{
	char buf[SDL_MAXSTRLEN];
	char *p;
	int xp = 0, yp = 0;

	vsnprintf (buf, SDL_MAXSTRLEN, fmt, ap);

	for (p=buf; *p != '\0'; p++)
	{
		if (*p == '\n')
		{
			yp++;
			xp = 0;
		}
		else if (*p == '\t')
		{
			xp += (8 - xp % 8);
		}
		else if (SDL_DrawChar(dest, x+xp*font->width, y+yp*font->height, font, *p))
			xp++;
	}
	
}

void SDL_PrintF(SDL_Surface *dest, int x, int y, SDL_Font *font, const char *szFormat, ...)
{
	va_list pArgs;
	char buf[SDL_MAXSTRLEN];
	char *p;
	int xp = 0, yp = 0;

	va_start (pArgs, szFormat);
	vsnprintf (buf, SDL_MAXSTRLEN, szFormat, pArgs);
	va_end (pArgs);

	for (p=buf; *p != '\0'; p++)
	{
		if (*p == '\n')
		{
			yp++;
			xp = 0;
		}
		else if (*p == '\t')
		{
			xp += (8 - xp % 8);
		}
		else if (SDL_DrawChar(dest, x+xp*font->width, y+yp*font->height, font, *p))
			xp++;
	}
}
